-- Intoduce the strategy to the host application (for example, Marketscope).
-- The function is called once when the host application initially loads the strategy.
function Init()
    -- User-friendly name and the description
    strategy:name(resources:get("param_Name")); 
    strategy:description(resources:get("param_Description"));

    -- Fast and slow moving average parameters

    strategy.parameters:addGroup(resources:get("param_SelectorGroup"));

    strategy.parameters:addString("Action".. 1, resources:get("param_Action1"), "", "SELL");
    strategy.parameters:addStringAlternative("Action".. 1, resources:get("param_NoAction"), "", "NO");
    strategy.parameters:addStringAlternative("Action".. 1, resources:get("param_SellAction"), "", "SELL"); 
    strategy.parameters:addStringAlternative("Action".. 1, resources:get("param_BuyAction"), "", "BUY");
    strategy.parameters:addStringAlternative("Action".. 1, resources:get("param_ClosePositionAction"), "", "CLOSE"); 

    strategy.parameters:addString("Action".. 2, resources:get("param_Action2"), "", "CLOSE");
    strategy.parameters:addStringAlternative("Action".. 2, resources:get("param_NoAction"), "", "NO");
    strategy.parameters:addStringAlternative("Action".. 2, resources:get("param_SellAction"), "", "SELL");
    strategy.parameters:addStringAlternative("Action".. 2, resources:get("param_BuyAction"), "", "BUY");
    strategy.parameters:addStringAlternative("Action".. 2, resources:get("param_ClosePositionAction"), "", "CLOSE");

    strategy.parameters:addString("Action".. 3, resources:get("param_Action3"), "", "NO");
    strategy.parameters:addStringAlternative("Action".. 3, resources:get("param_NoAction"), "", "NO");
    strategy.parameters:addStringAlternative("Action".. 3, resources:get("param_SellAction"), "", "SELL");
    strategy.parameters:addStringAlternative("Action".. 3, resources:get("param_BuyAction"), "", "BUY");
    strategy.parameters:addStringAlternative("Action".. 3, resources:get("param_ClosePositionAction"), "", "CLOSE");

    strategy.parameters:addString("Action".. 4, resources:get("param_Action4"), "", "NO");
    strategy.parameters:addStringAlternative("Action".. 4, resources:get("param_NoAction"), "", "NO");
    strategy.parameters:addStringAlternative("Action".. 4, resources:get("param_SellAction"), "", "SELL");
    strategy.parameters:addStringAlternative("Action".. 4, resources:get("param_BuyAction"), "", "BUY");
    strategy.parameters:addStringAlternative("Action".. 4, resources:get("param_ClosePositionAction"), "", "CLOSE");

    strategy.parameters:addString("Action".. 5, resources:get("param_Action5"), "", "CLOSE");
    strategy.parameters:addStringAlternative("Action".. 5, resources:get("param_NoAction"), "", "NO");
    strategy.parameters:addStringAlternative("Action".. 5, resources:get("param_SellAction"), "", "SELL");
    strategy.parameters:addStringAlternative("Action".. 5, resources:get("param_BuyAction"), "", "BUY");
    strategy.parameters:addStringAlternative("Action".. 5, resources:get("param_ClosePositionAction"), "", "CLOSE");

    strategy.parameters:addString("Action".. 6, resources:get("param_Action6"), "", "BUY");
    strategy.parameters:addStringAlternative("Action".. 6, resources:get("param_NoAction"), "", "NO");
    strategy.parameters:addStringAlternative("Action".. 6, resources:get("param_SellAction"), "", "SELL");
    strategy.parameters:addStringAlternative("Action".. 6, resources:get("param_BuyAction"), "", "BUY");
    strategy.parameters:addStringAlternative("Action".. 6, resources:get("param_ClosePositionAction"), "", "CLOSE");

    strategy.parameters:addGroup(resources:get("param_ParameterGroup"));
    strategy.parameters:addInteger("CCIP", resources:get("param_IndicatorPeriods"), "", 14, 2, 200);

    strategy.parameters:addGroup(resources:get("param_LevelsGroup"));
    strategy.parameters:addInteger("OB", resources:get("param_OverboughtLevel"), "", 100);
    strategy.parameters:addInteger("OS", resources:get("param_OversoldLevel"), "", -100);

    -- Price subscription parameters (bid or ask price, time frame)
    strategy.parameters:addGroup(resources:get("param_PriceGroup"));
    strategy.parameters:addString("PT", resources:get("R_PriceType"), "", "Bid");
    strategy.parameters:addStringAlternative("PT", resources:get("R_Bid"), "", "Bid");
    strategy.parameters:addStringAlternative("PT", resources:get("R_Ask"), "", "Ask");
    strategy.parameters:addString("TF", resources:get("param_TimeFrame"), "", "m1");
    strategy.parameters:setFlag("TF", core.FLAG_PERIODS);

    strategy.parameters:addGroup(resources:get("param_TradingParameters"));
    strategy.parameters:addBoolean("AllowTrade", resources:get("R_AllowTrade"), "", true);
    strategy.parameters:addString("MagicNumber", resources:get("param_MagicNumber"), "", "123456");

    strategy.parameters:addString("SIDE", resources:get("param_Side"), "", "BOTH");
    strategy.parameters:addStringAlternative("SIDE", resources:get("param_BothSide"), "", "BOTH");
    strategy.parameters:addStringAlternative("SIDE", resources:get("param_ShortSide"), "", "SHORT");
    strategy.parameters:addStringAlternative("SIDE", resources:get("param_LongSide"), "", "LONG");

    strategy.parameters:addString("Account", resources:get("R_Account2Trade"), "", "");
    strategy.parameters:setFlag("Account", core.FLAG_ACCOUNT);
    strategy.parameters:addInteger("Amount", resources:get("R_TradeAmountLots"), "", 1, 1, 100);
    strategy.parameters:addBoolean("SetLimit", resources:get("param_SetLimit_name"), "", true);
    strategy.parameters:addInteger("Limit", resources:get("param_Limit_name"), "", 30, 1, 10000);
    strategy.parameters:addBoolean("SetStop", resources:get("param_SetStop_name"), "", true);
    strategy.parameters:addInteger("Stop", resources:get("param_Stop_name"), "", 30, 1, 10000);
    strategy.parameters:addBoolean("TrailingStop", resources:get("param_TrailingStop_name"), "", false);

    strategy.parameters:addGroup(resources:get("R_SignalGroup"));
    strategy.parameters:addBoolean("ShowAlert", resources:get("R_ShowAlert"), "", true);
    strategy.parameters:addBoolean("PlaySound", resources:get("R_PlaySound"), "", false);
    strategy.parameters:addBoolean("RecurSound", resources:get("R_RecurrentSound"), "", false);
    strategy.parameters:addFile("SoundFile", resources:get("R_SoundFile"), "", "");
    strategy.parameters:setFlag("SoundFile", core.FLAG_SOUND);
    strategy.parameters:addBoolean("SendEmail", resources:get("R_SENDEMAIL"), "", false);
    strategy.parameters:addString("Email", resources:get("R_EMAILADDRESS"), resources:get("R_EMAILADDRESSDESCR"), "");
    strategy.parameters:setFlag("Email", core.FLAG_EMAIL);
end

-- The global variables
local BAR = nil;      -- the price history we subscribed for
local first;            -- the index of the oldest period where we can check whether moving averages has been crossed
local PlaySound;
local RecurrentSound;
local SoundFile;
local Email;
local SendEmail;
local AllowTrade;
local Account;
local Amount;
local BaseSize;
local SetLimit;
local Limit;
local SetStop;
local Stop;
local TrailingStop;
local Offer;
local CanClose;
local SIDE;

local name;

local OB,OS;
local CCI;
local Action={};

-- Prepare all the data.
-- The function is called once when the strategy is about to be started.
function Prepare(nameOnly)
    -- check moving average parameters
    OB=instance.parameters.OB;
    OS= instance.parameters.OS;
    SIDE= instance.parameters.SIDE;

    name = profile:id() .. ", " .. instance.bid:instrument();

    CCIP= instance.parameters.CCIP;
    local i;

    for i = 1, 6, 1 do
        Action[i]= instance.parameters:getString ("Action" .. i);
    end

    ShowAlert = instance.parameters.ShowAlert;
    if ShowAlert then
        PlaySound = instance.parameters.PlaySound;
        if PlaySound then
            RecurrentSound = instance.parameters.RecurSound;
            SoundFile = instance.parameters.SoundFile;
        else
            SoundFile = nil;
        end
        assert(not(PlaySound) or (PlaySound and SoundFile ~= ""), resources:get("R_SoundFileError"));
        SendEmail = instance.parameters.SendEmail;
        if SendEmail then
            Email = instance.parameters.Email;
        else
            Email = nil;
        end
        assert(not(SendEmail) or (SendEmail and Email ~= ""), resources:get("R_EmailAddressError"));
    end

    assert(instance.parameters.TF ~= "t1", resources:get("R_SrcErr"));
    -- check whether the strategy is allowed to trade
    AllowTrade = instance.parameters.AllowTrade;
    if AllowTrade then
        Account = instance.parameters.Account;
        Amount = instance.parameters.Amount;
        BaseSize = core.host:execute("getTradingProperty", "baseUnitSize", instance.bid:instrument(), Account);
        Offer = core.host:findTable("offers"):find("Instrument", instance.bid:instrument()).OfferID;
        CanClose = core.host:execute("getTradingProperty", "canCreateMarketClose", instance.bid:instrument(), Account);
        SetLimit = instance.parameters.SetLimit;
        Limit = instance.parameters.Limit;
        SetStop = instance.parameters.SetStop;
        Stop = instance.parameters.Stop;
        TrailingStop = instance.parameters.TrailingStop;
    end
    -- name the indicator
    instance:name(name);
    core.host:trace(GetTimeString(core.now()) .. " The Adaptable CCI strategy is implemented successfully in Time - Frame " .. instance.parameters.TF);
    -- setup the signal. pay attention, we pass "ShowAlert" (value initially taken from the instance.parameters.ShowAlert)
    -- here, so, we don't check whether alerts are requested anymore.
    -- and finally subscribe for the ticks of the instrument the user initially chosen to run the strategy for to
    -- have our strategy activated once.
    BAR = ExtSubscribe(1, nil, instance.parameters.TF,  instance.parameters.PT == "Bid", "bar");
    CCI = core.indicators:create("CCI", BAR, CCIP);
    first= CCI.DATA:first();
    if nameOnly then
        return ;
    end
end

-- the function is called every time when any subscribed price is changed. For tick subscribtions the function is called
-- for every tick, for the bar subscribtions the function is called when the candle is closed (in other words, when
-- the first tick of the next candle appears).
function ExtUpdate(id, source, period)
    if AllowTrade then
        if not(checkReady("trades")) or not(checkReady("orders")) then
            return ;
        end
    end
    if period < first + 1 then
        return;
    end
    CCI:update(core.UpdateLast);
    if core.crossesOver(CCI.DATA, OB, period) then
--        core.host:trace("");
--        core.host:trace(GetTimeString(core.now()) .. string.format(" The line Crosses OVER THE OVERBOUGHT Line, Condition met at CCI value = %f", CCI.DATA[period] ));
        ACTION(1);
    elseif core.crossesUnder(CCI.DATA, OB, period) then
--        core.host:trace("");
--        core.host:trace(GetTimeString(core.now()) .. string.format(" The line Crosses UNDER THE OVERBOUGHT Line, Condition met at CCI value = %f", CCI.DATA[period] ));
        ACTION(2);
    elseif core.crossesOver(CCI.DATA, 0, period) then
--        core.host:trace("");
--        core.host:trace(GetTimeString(core.now()) .. string.format(" The line Crosses OVER THE '0' Line, Condition met at CCI value = %f", CCI.DATA[period]));
        ACTION(3);
    elseif core.crossesUnder(CCI.DATA, 0, period) then
--        core.host:trace("");
--        core.host:trace(GetTimeString(core.now()) .. string.format(" The line Crosses UNDER THE '0' Line, Condition met at CCI value = %f", CCI.DATA[period]));
        ACTION(4);
    elseif core.crossesOver(CCI.DATA, OS, period) then
--        core.host:trace("");
--        core.host:trace(GetTimeString(core.now()) .. string.format(" The line Crosses OVER THE OVERSOLD Line, Condition met at CCI value = %f", CCI.DATA[period] ));
        ACTION(5);
    elseif core.crossesUnder(CCI.DATA, OS, period) then
--        core.host:trace("");
--        core.host:trace(GetTimeString(core.now()) .. string.format(" The line Crosses UNDER THE OVERSOLD Line, Condition met at CCI value = %f", CCI.DATA[period] ));
        ACTION(6);
    end
end

function ACTION (Flag)

    if Action[Flag] ==  "NO" then
--        core.host:trace(GetTimeString(core.now()) .. " Action set to - NO ACTION ");
        return;

    elseif Action[Flag] ==  "BUY" then
--        core.host:trace(GetTimeString(core.now()) .. " Action set to - BUY ");
        if haveTrades('B') then
            if haveTrades('S') then
--                core.host:trace(GetTimeString(core.now()) .. " Have a SELL Trade, Sending signal to Close Short ");
                exit('S');
                Signal ("Close Short");
            end
            return;
        end
        if SIDE == "SHORT" then
            if haveTrades('S') then
--                core.host:trace(GetTimeString(core.now()) .. " Side set to SHORT, Have a SELL Trade, Sending signal to Close Short ");
                exit('S');
                Signal ("Close Short");
            end
            return;
        end
        if AllowTrade then
            if haveTrades('S') then
--                core.host:trace(GetTimeString(core.now()) .. " Have a SELL Trade, Sending signal to Close Short ");
                exit('S');
                Signal ("Close Short");
            end
--            core.host:trace(GetTimeString(core.now()) .. " Sending signal to Open Long ");
            enter('B', Amount);
            Signal ("Open Long");
        else
--            core.host:trace(GetTimeString(core.now()) .. " Trade Not Allowed, Send signal to Close Short and Open Long ");
            Signal ("Close Short");
            Signal ("Open Long");
        end

    elseif Action[Flag] ==  "SELL" then

--        core.host:trace(GetTimeString(core.now()) .. " Action set to - SELL ");
        if haveTrades('S') then
            if haveTrades('B') then
--                core.host:trace(GetTimeString(core.now()) .. " Have a BUY Trade Open, Sending signal to CLOSE Long ");
                exit('B');
                Signal ("Close Long");
            end
            return;
        end
        if SIDE == "LONG" then
            if haveTrades('B') then
--                core.host:trace(GetTimeString(core.now()) .. " Side Set to LONG, Have a BUY Trade Open, Sending signal to Close Long ");
                exit('B');
                Signal ("Close Long");
            end
            return;
        end
        if AllowTrade then
            if haveTrades('B') then
--                core.host:trace(GetTimeString(core.now()) .. " Have a BUY Trade Open,  Sending signal to Close Long ");
                exit('B');
                Signal ("Close Long");
            end
--            core.host:trace(GetTimeString(core.now()) .. " Sending signal to Open Short ");
            enter('S', Amount);
            Signal ("Open Short");
        else
--            core.host:trace(GetTimeString(core.now()) .. " Trade Not Allowed, Send signal to Close Long and Open Short");
            Signal ("Close Long");
            Signal ("Open Short");
        end
    -- works fine -- CLOSE
    elseif Action[Flag] ==  "CLOSE" then
--        core.host:trace(GetTimeString(core.now()) .. " Action set to - CLOSE ");
        if AllowTrade then
            if haveTrades('B') then
--                core.host:trace(GetTimeString(core.now()) .. " Sending signal to Close Long ");
                exit('B');
                Signal ("Close Long");
            elseif haveTrades('S') then
--                core.host:trace(GetTimeString(core.now()) .. " Sending signal to Close short ");
                exit('S');
                Signal ("Close Short");
            else
--                core.host:trace(GetTimeString(core.now()) .. " No Trades available to Close ");
            end
        else
--            core.host:trace(GetTimeString(core.now()) .. " If have OPEN trades then Send signal to Close All ");
            Signal ("Close All");
        end
    end
end

--===========================================================================--
--                    TRADING UTILITY FUNCTIONS                              --
--===========================================================================--

function Signal (Label)
    if ShowAlert then
        terminal:alertMessage(instance.bid:instrument(), instance.bid[NOW],  Label, instance.bid:date(NOW));
    end
    if SoundFile ~= nil then
        terminal:alertSound(SoundFile, RecurrentSound);
    end
    if Email ~= nil then
        terminal:alertEmail(Email, Label, profile:id() .. "(" .. instance.bid:instrument() .. ")" .. instance.bid[NOW]..", " .. Label..", " .. instance.bid:date(NOW));
    end
end

function checkReady(table)
    local rc;
    if Account == "TESTACC_ID" then
        -- run under debugger/simulator
        rc = true;
    else
        rc = core.host:execute("isTableFilled", table);
    end
    return rc;
end

function tradesCount(BuySell)
    local enum, row;
    local count = 0;
    enum = core.host:findTable("trades"):enumerator();
    row = enum:next();
    while count == 0 and row ~= nil do
        if row.AccountID == Account and
            row.OfferID == Offer and
            (row.BS == BuySell or BuySell == nil) and
            row.QTXT == "Programming Services" .. " " .. profile:id() .. " --> " .. instance.parameters.MagicNumber then
            count = count + 1;
        end
        row = enum:next();
    end
    return count
end

function haveTrades(BuySell)
    local enum, row;
    local found = false;
    enum = core.host:findTable("trades"):enumerator();
    row = enum:next();
    while (not found) and (row ~= nil) do
        if row.AccountID == Account and
           row.OfferID == Offer and
           (row.BS == BuySell or BuySell == nil) and
           row.QTXT == "Programming Services" .. " " .. profile:id() .. " --> " .. instance.parameters.MagicNumber then
           found = true;
        end
        row = enum:next();
    end
    return found
end

-- enter into the specified direction
function enter(BuySell, Amount)
    if not(AllowTrade) then
        return true;
    end
    -- do not enter if position in the
    -- specified direction already exists
    if tradesCount(BuySell) > 0  then
        return true;
    end
    local valuemap, success, msg;
    valuemap = core.valuemap();
    valuemap.OrderType = "OM";
    valuemap.OfferID = Offer;
    valuemap.AcctID = Account;
    valuemap.Quantity = Amount * BaseSize;
    valuemap.BuySell = BuySell;
    valuemap.CustomID = "Programming Services" .. " " .. profile:id() .. " --> " .. instance.parameters.MagicNumber;

    -- add stop/limit
        valuemap.PegTypeStop = "O";
        if SetStop then
            if BuySell == "B" then
                valuemap.PegPriceOffsetPipsStop = -Stop;
            else
                valuemap.PegPriceOffsetPipsStop = Stop;
            end
        end
        if TrailingStop then
            valuemap.TrailStepStop = 1;
        end
        valuemap.PegTypeLimit = "O";
        if SetLimit then
            if BuySell == "B" then
                valuemap.PegPriceOffsetPipsLimit = Limit;
            else
                valuemap.PegPriceOffsetPipsLimit = -Limit;
            end
        end
        if (not CanClose) then
            valuemap.EntryLimitStop = 'Y'
        end
    success, msg = terminal:execute(100, valuemap);
    if not(success) then
        terminal:alertMessage(instance.bid:instrument(), instance.bid[instance.bid:size() - 1], "Open order failed" .. msg, instance.bid:date(instance.bid:size() - 1));
        return false;
    end
    return true;
end

-- exit from the specified direction
function exit(BuySell)
    if not(AllowTrade) then
        return true;
    end
    local valuemap, success, msg;
    if tradesCount(BuySell) > 0 then
        valuemap = core.valuemap();
        -- switch the direction since the order must be in oppsite direction
        if BuySell == "B" then
            BuySell = "S";
        else
            BuySell = "B";
        end
        valuemap.OrderType = "CM";
        valuemap.OfferID = Offer;
        valuemap.AcctID = Account;
        valuemap.NetQtyFlag = "Y";
        valuemap.BuySell = BuySell;
        valuemap.CustomID = "Programming Services" .. " " .. profile:id() .. " --> " .. instance.parameters.MagicNumber;
        success, msg = terminal:execute(101, valuemap);
        if not(success) then
            terminal:alertMessage(instance.bid:instrument(), instance.bid[instance.bid:size() - 1], "Open order failed" .. msg, instance.bid:date(instance.bid:size() - 1));
            return false;
        end
        return true;
    end
    return false;
end

function GetTimeString(date)
    local dateTable = core.dateToTable(date);
    local str = string.format("%02i/%02i/%04i  %02i:%02i:%02i",dateTable.month, dateTable.day, dateTable.year, dateTable.hour, dateTable.min, dateTable.sec);
    return str;
end

dofile(core.app_path() .. "\\strategies\\standard\\include\\helper.lua");
