-- Strategy profile initialization routine
-- Defines Strategy profile properties and Strategy parameters
-- TODO: Add minimal and maximal value of numeric parameters and default color of the streams
function Init()
    strategy:name(resources:get("param_Name"));
    strategy:description(resources:get("param_Description"));
    strategy.parameters:addGroup(resources:get("R_ParamGroup"));
    strategy.parameters:addString("TF_RSI", resources:get("param_TF_RSI"), resources:get("param_TF_RSI_desc"), "m1");
    strategy.parameters:setFlag("TF_RSI", core.FLAG_PERIODS);
    strategy.parameters:addInteger("RSIPeriod", resources:get("param_RSIPeriod"), resources:get("param_RSIPeriod_desc"), 14);
    strategy.parameters:addInteger("RSIOverbought", resources:get("param_RSIOverbought"), resources:get("param_RSIOverbought_desc"), 70);
    strategy.parameters:addInteger("RSIOversold", resources:get("param_RSIOversold"), resources:get("param_RSIOversold_desc"), 30);
    --strategy.parameters:addString("Side", "Side", "No description", "B");
    --strategy.parameters:addStringAlternative("Side", "Buy", "", "B");
    --strategy.parameters:addStringAlternative("Side", "Sell", "", "S");
    strategy.parameters:addInteger("LotSize", resources:get("R_TradeAmountLots"), "", 1);
    strategy.parameters:addInteger("GridSpacingPips", resources:get("param_GridSpacingPips"),resources:get("param_GridSpacingPips_desc"), 20);
    strategy.parameters:addInteger("TakeProfit", resources:get("param_TakeProfit"), resources:get("param_TakeProfit_desc"), 20);
    strategy.parameters:addInteger("MaxTradesPerSession", resources:get("param_MaxTradesPerSession"), resources:get("param_MaxTradesPerSession_desc"), 20);
    strategy.parameters:addString("StartTime", resources:get("param_StartTime"), resources:get("param_StartTime_desc"), "06:00");
    strategy.parameters:addString("StopTime", resources:get("param_StopTime"), resources:get("param_StopTime_desc"), "14:00");
    strategy.parameters:addString("MagicNumber", resources:get("param_MagicNumber"), "", "123456");
    strategy.parameters:addBoolean("Debug", resources:get("param_Debug"), resources:get("param_Debug_desc"), false);

    strategy.parameters:addGroup(resources:get("R_Trading_ParamGroup"));
    strategy.parameters:addString("Account", resources:get("R_Account2Trade"), "", "");
    strategy.parameters:setFlag("Account", core.FLAG_ACCOUNT);
end


local LotSize;
local RSIPeriod;
local RSIOverbought;
local RSIOversold;
local GridSpacingPips;
local TakeProfit;
local MaxTradesPerSession;
local StartTime;
local StopTime;
local MagicNumber;
local Debug;

local gSource = nil;
local barSource = nil;
local AllowTicks = true;

local Account;
local Amount;
local BaseSize;
local Offer;
local CanClose;
local CustomTXT;
local CustomFirstTXT;
local fileName = "";
local File;
local iRSI;

local levelPrice;
local firstOrderSentForTheDay;
local firstOrderSide;
local tradeCount;

-- Routine
function Prepare(nameOnly)
    LotSize = instance.parameters.LotSize;
    RSIPeriod = instance.parameters.RSIPeriod;
    RSIOverbought = instance.parameters.RSIOverbought;
    RSIOversold = instance.parameters.RSIOversold;
    GridSpacingPips = instance.parameters.GridSpacingPips;
    TakeProfit = instance.parameters.TakeProfit;
    MaxTradesPerSession = instance.parameters.MaxTradesPerSession;
    StartTime = instance.parameters.StartTime;
    StopTime = instance.parameters.StopTime;
    MagicNumber = instance.parameters.MagicNumber;
    Debug = instance.parameters.Debug;

    local name = profile:id() .. "(" .. instance.bid:instrument()  .. ", " .. tostring(LotSize) .. ", " .. tostring(RSIPeriod) .. ", " .. tostring(RSIOverbought).. ", " .. tostring(RSIOversold).. ", " .. tostring(GridSpacingPips).. ", " .. tostring(TakeProfit) .. ", " .. tostring(MaxTradesPerSession) .. ", " .. tostring(StartTime) .. ", " .. tostring(StopTime) .. ", " .. tostring(MagicNumber) .. ", " .. tostring(Debug) .. ")";
    instance:name(name);

    if nameOnly then
        return ;
    end

    if (not(AllowTicks)) then
        assert(instance.parameters.TF ~= "t1", resources:get("R_SrcErr"));
    end

    Account = instance.parameters.Account;
    Amount = instance.parameters.Amount;
    BaseSize = core.host:execute("getTradingProperty", "baseUnitSize", instance.bid:instrument(), Account);
    Offer = core.host:findTable("offers"):find("Instrument", instance.bid:instrument()).OfferID;
    CanClose = core.host:execute("getTradingProperty", "canCreateMarketClose", instance.bid:instrument(), Account);
    TakeProfit = instance.parameters.TakeProfit * instance.bid:pipSize();

    barSource = ExtSubscribe(1, nil, instance.parameters.TF_RSI, true, "bar");
    gSource = ExtSubscribe(2, nil, "t1", true, "bar");
    first = barSource:first();
    --TODO: Find indicator's profile, intialize parameters, and create indicator's instance (if needed)
    CustomTXT = "PSS_AsiaRangeGrid_"..MagicNumber;
    CustomFirstTXT = "PSS_AsiaRangeGrid_First_"..MagicNumber;

    local cur = instance.bid:instrument();
    local pos = string.find(cur, "/");
    if pos ~= nil then
        cur = string.sub(cur, 0, pos-1)..string.sub(cur, pos+1);
    end
    
    iRSI = core.indicators:create("RSI", barSource.close, RSIPeriod);
    firstOrderSentForTheDay = false;
    tradeCount = 0;
    levelPrice = 0;

    if instance.parameters.Debug == true then    
        fileName = tostring("c:\\Marketscope Logs\\" .. profile:id().."_"..tostring(cur).."_"..tostring(MagicNumber)..".txt");
        core.host:trace("fileName: " .. fileName);
        os.execute('MKDIR "'.."c://Marketscope Logs//"..'"');
   end
   printToFile(name .. "----------------------------------------------------");
end

-- strategy calculation routine
-- TODO: Add your code for decision making
-- TODO: Update the instance of your indicator(s) if needed
function ExtUpdate(id, source, period)
    if id == 1 then
        iRSI:update(core.UpdateLast);
        local RSI = iRSI.RSI;
        if IsTradingTime(source, period) then
            if firstOrderSentForTheDay == false and MaxTradesPerSession > 0 then
                printToFile(core.formatDate(source:date(period)).." CurrentRSI: " .. tostring(RSI[period]));
                if period > first and source:hasData(period) and RSI:hasData(period) and RSI:hasData(period-2) then
                    if core.crossesOver(RSI, RSIOversold, period) then
                        printToFile(core.formatDate(source:date(period)).." RSI crossed over oversold"..tostring(RSIOversold).." RSI[period]:" .. tostring(RSI[period]).." RSI[period-1]:" .. tostring(RSI[period-1]).." RSI[period-2]:" .. tostring(RSI[period-2]));
                        --Open buy
                        tradeCount = tradeCount + 1;
                        entry("B", CustomFirstTXT);
                        firstOrderSentForTheDay = true;
                    elseif core.crossesUnder(RSI, RSIOverbought, period) then
                        printToFile(core.formatDate(source:date(period)).." RSI crossed under overbought"..tostring(RSIOverbought).." RSI[period]:" .. tostring(RSI[period]).." RSI[period-1]:" .. tostring(RSI[period-1]).." RSI[period-2]:" .. tostring(RSI[period-2]));
                        --Open sell
                        tradeCount = tradeCount + 1;
                        entry("S", CustomFirstTXT);
                        firstOrderSentForTheDay = true;
                    end
                end
            end
        end
    end
    
    if id == 2 then
        if IsTradingTime(source, period) then
            --Grid logic
            if (firstOrderSentForTheDay == true and levelPrice > 0) and MaxTradesPerSession > tradeCount then
                local Bid = instance.bid[NOW];
                if (firstOrderSide == "S" and Bid >= levelPrice + GridSpacingPips * instance.bid:pipSize()) then
                    levelPrice = levelPrice + GridSpacingPips * instance.bid:pipSize();
                    --more sell
                    tradeCount = tradeCount + 1;
                    entry("S", CustomTXT);
                    printToFile(core.formatDate(source:date(period)).." New LevelPrice: " .. tostring(levelPrice));
                end
                if (firstOrderSide == "B" and Bid <= levelPrice - GridSpacingPips * instance.bid:pipSize()) then
                    levelPrice = levelPrice - GridSpacingPips * instance.bid:pipSize();
                    --more buys
                    tradeCount = tradeCount + 1;
                    entry("B", CustomTXT);
                    printToFile(core.formatDate(source:date(period)).." New LevelPrice: " .. tostring(levelPrice));
                end
            end
        else
            -- close trades
            if firstOrderSentForTheDay == true then
                printToFile(core.formatDate(source:date(period)).."   Expiration time reached.");
                firstOrderSentForTheDay = false;
                CloseAll();
                tradeCount = 0;
                levelPrice = 0;
            end
        end
    end
end


function entry(BuySell, custTXT)
    local valuemap, success, msg;

    valuemap = core.valuemap();

    valuemap.OrderType = "OM";
    valuemap.OfferID = Offer;
    valuemap.AcctID = Account;
    valuemap.Quantity = LotSize * BaseSize;
    valuemap.BuySell = BuySell;
    valuemap.PegTypeStop = "M";
    valuemap.GTC = "GTC";
    valuemap.CustomID = custTXT;
    
    if TakeProfit > 0 then
        -- set limit order
        if BuySell == "B" then
            valuemap.RateLimit = instance.ask[NOW] + TakeProfit;
        else
            valuemap.RateLimit = instance.bid[NOW] - TakeProfit;
        end
    end

    if (not CanClose) and (TakeProfit > 0) then
        valuemap.EntryLimitStop = 'Y'
    end
    
    success, msg = terminal:execute(100, valuemap);

    if not(success) then
        terminal:alertMessage(instance.bid:instrument(), instance.bid[instance.bid:size() - 1], "alert_OpenOrderFailed: " .. msg, instance.bid:date(instance.bid:size() - 1));
        return false;
    end

    return true;
end

-- exit from the specified direction
function CloseAll()
    local enum, trade;
    enum = core.host:findTable("trades"):enumerator();
    trade = enum:next();
    if CanClose == true then
        while (trade ~= nil) do
            if trade.AccountID == Account and
               trade.OfferID == Offer and 
               trade.QTXT == CustomTXT or trade.QTXT == CustomFirstTXT then
                local valuemap;
                valuemap = core.valuemap();
                valuemap.TradeID = trade.TradeID;
                valuemap.OrderType = "CM";
                valuemap.OfferID = trade.OfferID;
                valuemap.AcctID = trade.AccountID;
                valuemap.Quantity = trade.Lot;
                valuemap.CustomID = trade.QTXT;
                if trade.BS == "B" then valuemap.BuySell = "S"; else valuemap.BuySell = "B"; end
                success, msg = terminal:execute(200, valuemap);
                assert(success, msg);
                if not(success) then
                    terminal:alertMessage(instance.bid:instrument(), instance.bid[instance.bid:size() - 1], "alert_OpenOrderFailed" .. msg, instance.bid:date(instance.bid:size() - 1));
                end
                printToFile(core.formatDate(core.now()).."   Closing "..tostring(trade.BS).." trade");
            end
            trade = enum:next();
        end
    else
        local found = false;
        local BuySell = "R";
        while (trade ~= nil) do
            if trade.AccountID == Account and
               trade.OfferID == Offer and 
               trade.QTXT == CustomTXT or trade.QTXT == CustomFirstTXT then
                found = true;
                BuySell = trade.BS;
            end
            trade = enum:next();
        end
        if found == true then
            valuemap = core.valuemap();
            -- switch the direction since the order must be in oppsite direction
            if BuySell == "B" then
                BuySell = "S";
            elseif BuySell == "S" then
                BuySell = "B";
            end
            valuemap.OrderType = "CM";
            valuemap.OfferID = Offer;
            valuemap.AcctID = Account;
            valuemap.NetQtyFlag = "Y";
            valuemap.BuySell = BuySell;
            valuemap.GTC = "GTC";
            valuemap.CustomID = CustomTXT;

            success, msg = terminal:execute(201, valuemap);

            if not(success) then
                terminal:alertMessage(instance.bid:instrument(), instance.bid[instance.bid:size() - 1], "alert_OpenOrderFailed" .. msg, instance.bid:date(instance.bid:size() - 1));
            end
            printToFile(core.formatDate(core.now()).."   Closing All sent. BuySell: "..tostring(BuySell));
        end
    end
end

--Converts a time in string to time value
function ConvertStrToTime(timeString, source, period)
    local h, m;
    local currentTime = source:date(NOW);
    
    local nowTable = core.dateToTable(currentTime);

    for HH, MM in string.gmatch(timeString, "(%w+):(%w+)") do
        h = tonumber(HH);
        m = tonumber(MM);
    end
    local time = core.datetime(nowTable.year, nowTable.month, nowTable.day, h, m, 0);
    return time;
end

--return true if current time is in trading time
function IsTradingTime(source, period)
    local currentTime = source:date(NOW);
    local startTimeToday = ConvertStrToTime(StartTime, source, period);
    local endTimeToday = ConvertStrToTime(StopTime, source, period);
    --printToFile("CurrentTime: " .. core.formatDate(currentTime).." StartTime: "..core.formatDate(startTimeToday).." EndTime: "..core.formatDate(endTimeToday));
    
    --compare current time
    if startTimeToday <= endTimeToday then
        if currentTime >= startTimeToday and currentTime <= endTimeToday then
            return true;
        else
            return false;
        end
    else
        if currentTime >= startTimeToday or currentTime <= endTimeToday then
            return true;
        else
            return false;
        end
    end
end


function ExtAsyncOperationFinished(cookie, success, message)
    if cookie == 100 then
        if success then
            --terminal:alertMessage(instance.bid:instrument(), instance.bid[NOW], "Order Filled.", instance.bid:date(NOW));
            printToFile("cookie: " .. tostring(cookie) .. " success: " .. tostring(success) .. " message: " .. tostring(message));
            local enum, trade;
            enum = core.host:findTable("trades"):enumerator();
            trade = enum:next();
            while (trade ~= nil) do
                if trade.AccountID == Account and
                   trade.OfferID == Offer and 
                   trade.QTXT == CustomFirstTXT and
                   levelPrice == 0 then
                    levelPrice = trade.Open;
                    firstOrderSide = trade.BS;
                    printToFile("First trade.Open: " .. tostring(trade.Open) .. " trade.BS: " .. tostring(trade.BS));
                end
                trade = enum:next();
            end
        else
            --terminal:alertMessage(instance.bid:instrument(), instance.bid[NOW], "Order failed."..message, instance.bid:date(NOW));
        end
   end
end

function printToFile(message)
    if instance.parameters.Debug == true then
        --try
            File = io.open(fileName, "a");
            File:write(message.."\n");
            File:flush();
            File:close();
        --catch err do
        --    core.host:trace("An error occured: "..err);
        --end
    end
    core.host:trace(message);
end

dofile(core.app_path() .. "\\strategies\\standard\\include\\helper.lua");