-- Updated December 2015 by Rob Pasche

function Init()
    strategy:name(resources:get("param_Name"));
    strategy:description(resources:get("param_Description"));
    strategy:type(core.Strategy);

    strategy.parameters:addGroup(resources:get("R_Trading_ParamGroup"));
    strategy.parameters:addString("TicketID", resources:get("param_TicketID"), "", "");
    strategy.parameters:setFlag("TicketID", core.FLAG_TRADE);
    strategy.parameters:addString("Account", resources:get("R_Account2Trade"), "", "");
    strategy.parameters:setFlag("Account", core.FLAG_ACCOUNT);
    strategy.parameters:addString("OffsetType", resources:get("param_OffsetType"), resources:get("param_OffsetType_desc"), "TradeOpenPrice");
    strategy.parameters:addStringAlternative("OffsetType", resources:get("param_OffsetType_1"), resources:get("param_OffsetType_1_desc"), "TradeOpenPrice");
    strategy.parameters:addStringAlternative("OffsetType", resources:get("param_OffsetType_2"), resources:get("param_OffsetType_2_desc"), "CurrentMarketPrice");
    strategy.parameters:addStringAlternative("OffsetType", resources:get("param_OffsetType_3"), resources:get("param_OffsetType_3_desc"), "NoOffset");

    strategy.parameters:addGroup(resources:get("param_LimitStop1Group"));
    strategy.parameters:addDouble("Limit1", resources:get("param_Limit1"),resources:get("param_Limit1_desc"), 10);
    strategy.parameters:addDouble("Stop1", resources:get("param_Stop1"),resources:get("param_Stop1_desc"), -10);
    strategy.parameters:addInteger("Amount1", resources:get("param_Amount1"),resources:get("param_Amount1_desc"), 1);

    strategy.parameters:addGroup(resources:get("param_LimitStop2Group"));
    strategy.parameters:addDouble("Limit2", resources:get("param_Limit2"),resources:get("param_Limit2_desc"), 20);
    strategy.parameters:addDouble("Stop2", resources:get("param_Stop2"),resources:get("param_Stop2_desc"), -20);
    strategy.parameters:addInteger("Amount2", resources:get("param_Amount2"),resources:get("param_Amount2_desc"), 2);

    strategy.parameters:addGroup(resources:get("param_LimitStop3Group"));
    strategy.parameters:addDouble("Limit3", resources:get("param_Limit3"),resources:get("param_Limit3_desc"), 30);
    strategy.parameters:addDouble("Stop3", resources:get("param_Stop3"),resources:get("param_Stop3_desc"), -30);
    strategy.parameters:addInteger("Amount3", resources:get("param_Amount3"),resources:get("param_Amount3_desc"), 3);
    
    strategy.parameters:addGroup(resources:get("param_MiscGroup"));
    strategy.parameters:addBoolean("Debug", resources:get("param_Debug"), "", true);

end
-- Trading parameters
local Account = nil;
local TicketID = nil;
local Trade;

-- parameters
local Limit1 = nil;
local Limit2 = nil;
local Limit3 = nil;
local Stop1 = nil;
local Stop2 = nil;
local Stop3 = nil;
local Amount1 = nil;
local Amount2 = nil;
local Amount3 = nil;

local StopLevel1;
local StopLevel2;
local StopLevel3;
local LimitLevel1;
local LimitLevel2;
local LimitLevel3;

local StopLimit1Valid = true;
local StopLimit2Valid = true;
local StopLimit3Valid = true;

local Direction;

local StopLimitLevelsSet = false;

local Debug;
local gSource = nil; -- the source stream

local Account;
local BaseSize;
local Offer;
local CanClose;
local PointSize;
local Type;

local Executing;
local logger;
local LogTag = true;


function Prepare(nameOnly)
    TicketID = instance.parameters.TicketID;
    Account = instance.parameters.Account;
    OffsetType = instance.parameters.OffsetType;
    Limit1 = instance.parameters.Limit1;
    Limit2 = instance.parameters.Limit2;
    Limit3 = instance.parameters.Limit3;
    Stop1 = instance.parameters.Stop1;
    Stop2 = instance.parameters.Stop2;
    Stop3 = instance.parameters.Stop3;
    Amount1 = instance.parameters.Amount1;
    Amount2 = instance.parameters.Amount2;
    Amount3 = instance.parameters.Amount3;
    Debug = instance.parameters.Debug;


    local name = profile:id() .. "(" .. instance.bid:instrument() .. ", " .. tostring(OffsetType) .. ", " .. tostring(Limit1) .. ", " .. tostring(Stop1) .. ", " .. tostring(Amount1) .. ", " .. tostring(Limit2) .. ", " .. tostring(Stop2) .. ", " .. tostring(Amount2) .. ", " .. tostring(Limit3) .. ", " .. tostring(Stop3) .. ", " .. tostring(Amount3) .. ", " .. tostring(Debug) .. ")";
    instance:name(name);

    if nameOnly then
        return ;
    end

    BaseSize = core.host:execute("getTradingProperty", "baseUnitSize", instance.bid:instrument(), Account);
    Offer = core.host:findTable("offers"):find("Instrument", instance.bid:instrument()).OfferID;
    CanClose = core.host:execute("getTradingProperty", "canCreateMarketClose", instance.bid:instrument(), Account);
    PointSize = core.host:findTable("offers"):find("Instrument", instance.bid:instrument()).PointSize;
    Type = core.host:findTable("offers"):find("Instrument", instance.bid:instrument()).InstrumentType;

    gSource = ExtSubscribe(1, nil, "t1", true, "bar"); 
    --TODO: Find indicator's profile, intialize parameters, and create indicator's instance (if needed)
    if Trade ~= nil then     
    Trade = core.host:findTable("trades"):find("TradeID", TicketID)
    
    Direction = Trade.BS
    
    assert(Trade.Instrument == instance.bid:instrument(), "Error: Attempting to run strategy on symbol " .. tostring(instance.bid:instrument()) .. ". That is different than the selected trade's symbol, " .. tostring(Trade.Instrument) .. ". Please change 'Symbol' in the parameters to match the trade's symbol.");

    instrument = string.gsub(tostring(instance.bid:instrument()), "/", "");
        if Debug then
            logger = Logger:create("FXCM-Multiple-Stops-Limits-"..instrument, Logger.DEBUG);
        else
            logger = Logger:create("FXCM-Multiple-Stops-Limits-"..instrument, Logger.INFO);
        end
    end
end



function ExtUpdate(id, source, period)
   if Trade ~= nil then
    if LogTag then
        logger:info(core.formatDate(source:date(period)).."   Instrument:" .. tostring(instance.bid:instrument()) ..  "  OffsetType:" .. tostring(OffsetType) .. "  Limit1:" .. tostring(Limit1)  .. "  Stop1:" .. tostring(Stop1) .. "  Amount1:" .. tostring(Amount1) .. "  Limit2:" .. tostring(Limit2)  .. "  Stop2:" .. tostring(Stop2) .. "  Amount2:" .. tostring(Amount2) .. "  Limit3:" .. tostring(Limit3)  .. "  Stop3:" .. tostring(Stop3) .. "  Amount3:" .. tostring(Amount3) .. "  Debug:" .. tostring(Debug));
        logger:info(core.formatDate(source:date(period)).."   Starting Strategy.");
        LogTag = false
        
        -- calculate values for StopLevel1, LimitLevel1, etc... also invalidates when amount = 0
        setStopLimitLevels();
        
        core.host:trace("Stop1: " .. tostring(StopLevel1) .. "   Stop2: " .. tostring(StopLevel2) .. "   Stop3: " .. tostring(StopLevel3) .. "   Limit1: " .. tostring(LimitLevel1) .. "   Limit2: " .. tostring(LimitLevel2) .. "   Limit3: " .. tostring(LimitLevel3))
        logger:info(core.formatDate(source:date(period)).."Stop1: " .. tostring(StopLevel1) .. "   Stop2: " .. tostring(StopLevel2) .. "   Stop3: " .. tostring(StopLevel3) .. "   Limit1: " .. tostring(LimitLevel1) .. "   Limit2: " .. tostring(LimitLevel2) .. "   Limit3: " .. tostring(LimitLevel3))
        
        -- ensures that stop and limits levels are correct in respect to current market prices
        checkValidity();
        
        -- displays message with stop/limit levels
        terminal:alertMessage (instance.bid:instrument(), 0, "Stop and Limit values are valid. Managing positions...                                                                                                                                 Stop/Limit1..." .. tostring(StopLevel1) .. "/" .. tostring(LimitLevel1) .. "     will close ".. tostring(Amount1) .. " lot(s)                Stop/Limit2..." .. tostring(StopLevel2) .. "/" .. tostring(LimitLevel2) .. "     will close ".. tostring(Amount2) .. " lot(s)                Stop/Limit3..." .. tostring(StopLevel3) .. "/" .. tostring(LimitLevel3) .. "     will close ".. tostring(Amount3) .. " lot(s)                         ", source:date(period))
        
    end
    
    --core.host:trace("Test inside Update... Bid Price: " .. instance.bid[NOW] .. "   Ask Price: " .. instance.ask[NOW])
    
    
    if not Executing and StopLimitLevelsSet then
    
    
        -- Stop/Limit #1
        if StopLimit1Valid then
            
            -- if buy trade
            if Direction == "B" then
                if instance.bid[NOW] <= StopLevel1 then
                    StopLimit1Valid = false
                    Executing = true
                    core.host:trace("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Stop1 (" .. tostring(StopLevel1) .. ").")
                    logger:info("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Stop1 (" .. tostring(StopLevel1) .. ").")
                    exit(Amount1);
                    return;
                end
                
                if instance.bid[NOW] >= LimitLevel1 then
                    StopLimit1Valid = false
                    Executing = true
                    core.host:trace("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Limit1 (" .. tostring(LimitLevel1) .. ").")
                    logger:info("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Limit1 (" .. tostring(LimitLevel1) .. ").")
                    exit(Amount1);
                    return;
                end
            end
            
            -- if sell trade
            if Direction == "S" then
                if instance.ask[NOW] >= StopLevel1 then
                    StopLimit1Valid = false
                    Executing = true
                    core.host:trace("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Stop1 (" .. tostring(StopLevel1) .. ").")
                    logger:info("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Stop1 (" .. tostring(StopLevel1) .. ").")
                    exit(Amount1);
                    return;
                end
                
                if instance.ask[NOW] <= LimitLevel1 then
                    StopLimit1Valid = false
                    Executing = true
                    core.host:trace("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Limit1 (" .. tostring(LimitLevel1) .. ").")
                    logger:info("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Limit1 (" .. tostring(LimitLevel1) .. ").")
                    exit(Amount1);
                    return;
                end
            end
            
        end
        
        -- Stop/Limit #2
        if StopLimit2Valid then
        
            -- if buy trade
            if Direction == "B" then
                if instance.bid[NOW] <= StopLevel2 then
                    StopLimit2Valid = false
                    Executing = true
                    core.host:trace("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Stop2 (" .. tostring(StopLevel2) .. ").")
                    logger:info("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Stop2 (" .. tostring(StopLevel2) .. ").")
                    exit(Amount2);
                    return;
                end
                
                if instance.bid[NOW] >= LimitLevel2 then
                    StopLimit2Valid = false
                    Executing = true
                    core.host:trace("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Limit2 (" .. tostring(LimitLevel2) .. ").")
                    logger:info("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Limit2 (" .. tostring(LimitLevel2) .. ").")
                    exit(Amount2);
                    return;
                end
            end
            
            -- if sell trade
            if Direction == "S" then
                if instance.ask[NOW] >= StopLevel2 then
                    StopLimit2Valid = false
                    Executing = true
                    core.host:trace("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Stop2 (" .. tostring(StopLevel2) .. ").")
                    logger:info("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Stop2 (" .. tostring(StopLevel2) .. ").")
                    exit(Amount2);
                    return;
                end
                
                if instance.ask[NOW] <= LimitLevel2 then
                    StopLimit2Valid = false
                    Executing = true
                    core.host:trace("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Limit2 (" .. tostring(LimitLevel2) .. ").")
                    logger:info("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Limit2 (" .. tostring(LimitLevel2) .. ").")
                    exit(Amount2);
                    return;
                end
            end
        
        end
        
        -- Stop/Limit #3
        if StopLimit3Valid then
        
            -- if buy trade
            if Direction == "B" then
                if instance.bid[NOW] <= StopLevel3 then
                    StopLimit3Valid = false
                    Executing = true
                    core.host:trace("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Stop3 (" .. tostring(StopLevel3) .. ").")
                    logger:info("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Stop3 (" .. tostring(StopLevel3) .. ").")
                    exit(Amount3);
                    return;
                end
                
                if instance.bid[NOW] >= LimitLevel3 then
                    StopLimit3Valid = false
                    Executing = true
                    core.host:trace("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Limit3 (" .. tostring(LimitLevel3) .. ").")
                    logger:info("Bid Price (" .. tostring(instance.bid[NOW]) .. ") has triggered Limit3 (" .. tostring(LimitLevel3) .. ").")
                    exit(Amount3);
                    return;
                end
            end
            
            -- if sell trade
            if Direction == "S" then
                if instance.ask[NOW] >= StopLevel3 then
                    StopLimit3Valid = false
                    Executing = true
                    core.host:trace("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Stop3 (" .. tostring(StopLevel3) .. ").")
                    logger:info("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Stop3 (" .. tostring(StopLevel3) .. ").")
                    exit(Amount3);
                    return;
                end
                
                if instance.ask[NOW] <= LimitLevel3 then
                    StopLimit3Valid = false
                    Executing = true
                    core.host:trace("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Limit3 (" .. tostring(LimitLevel3) .. ").")
                    logger:info("Ask Price (" .. tostring(instance.ask[NOW]) .. ") has triggered Limit3 (" .. tostring(LimitLevel3) .. ").")
                    exit(Amount3);
                    return;
                end
            end
        
        end
    
        -- Stop Strategy when all have executed
        if not StopLimit1Valid and not StopLimit2Valid and not StopLimit3Valid then
            core.host:trace("All stops/limits executed, stopping strategy.");
            logger:info(core.formatDate(source:date(period)).."All saved stops/limits executed, stopping strategy.");
            terminal:alertMessage (instance.bid:instrument(), 0, "All saved stops/limits executed, stopping strategy.", source:date(period))
            -- stop strategy
            core.host:execute("stop");
        end
    
    
        else
            core.host:trace("Processing order... waiting until complete...");
        end
    else
        core.host:execute("stop");
    end
end

-- only used for UK accounts to check TicketID
function checkTradeID()

    local TicketExists = false;
    
    -- checks to see if original TicketID exists as open trade
    local enum, row;
    local valid = false;
    enum = core.host:findTable("trades"):enumerator();
    row = enum:next()
    while row ~= nil do
        if row.TradeID == TicketID then
            Trade = core.host:findTable("trades"):find("TradeID", TicketID);
            TicketExists = true;
            row = nil;
            valid = true;
        end
        row = enum:next();
    end

    -- if TicketID no longer exists, change Trade to ticket with TradeIDOrigin that = TicketID
    if not valid then
        local enum2, row2
        enum2 = core.host:findTable("trades"):enumerator();
        row2 = enum2:next()
        while row2 ~= nil do
            if row2.TradeIDOrigin == TicketID then
                Trade = core.host:findTable("trades"):find("TradeID", row2.TradeID);
                TicketID = Trade.TradeID; -- new reference TicketID for the next trade
                TicketExists = true;
                row2 = nil;
            end
            row2 = enum2:next();
        end
    end
    
    assert(TicketExists, "Error: There is no trade that matches the TicketID or originates from TicketID. Stopping Strategy.");


end


-- calculates values for StopLevel1, LimitLevel1, etc... also invalidates when amount == 0
function setStopLimitLevels()

    if OffsetType == "TradeOpenPrice" then
    
        if Amount1 > 0 then
            if Direction == "B" then
                StopLevel1 = Trade.Open + PointSize*Stop1;
                LimitLevel1 = Trade.Open + PointSize*Limit1;
            else
                StopLevel1 = Trade.Open - PointSize*Stop1;
                LimitLevel1 = Trade.Open - PointSize*Limit1;
            end
        else
            StopLimit1Valid = false
        end
        
        if Amount2 > 0 then
            if Direction == "B" then
                StopLevel2 = Trade.Open + PointSize*Stop2;
                LimitLevel2 = Trade.Open + PointSize*Limit2;
            else
                StopLevel2 = Trade.Open - PointSize*Stop2;
                LimitLevel2 = Trade.Open - PointSize*Limit2;
            end
        else
            StopLimit2Valid = false
        end
        
        if Amount3 > 0 then
            if Direction == "B" then
                StopLevel3 = Trade.Open + PointSize*Stop3;
                LimitLevel3 = Trade.Open + PointSize*Limit3;
            else
                StopLevel3 = Trade.Open - PointSize*Stop3;
                LimitLevel3 = Trade.Open - PointSize*Limit3;
            end
        else
            StopLimit3Valid = false
        end
    
    elseif OffsetType == "CurrentMarketPrice" then
        
        if Amount1 > 0 then
            if Direction == "B" then
                StopLevel1 = instance.bid[NOW]+PointSize*Stop1
                LimitLevel1 = instance.bid[NOW]+PointSize*Limit1
            else
                StopLevel1 = instance.ask[NOW]-PointSize*Stop1
                LimitLevel1 = instance.ask[NOW]-PointSize*Limit1
            end
        else
            StopLimit1Valid = false
        end
        
        if Amount2 > 0 then
            if Direction == "B" then
                StopLevel2 = instance.bid[NOW]+PointSize*Stop2
                LimitLevel2 = instance.bid[NOW]+PointSize*Limit2
            else
                StopLevel2 = instance.ask[NOW]-PointSize*Stop2
                LimitLevel2 = instance.ask[NOW]-PointSize*Limit2
            end
        else
            StopLimit2Valid = false
        end
        
        if Amount3 > 0 then
            if Direction == "B" then
                StopLevel3 = instance.bid[NOW]+PointSize*Stop3
                LimitLevel3 = instance.bid[NOW]+PointSize*Limit3
            else
                StopLevel3 = instance.ask[NOW]-PointSize*Stop3
                LimitLevel3 = instance.ask[NOW]-PointSize*Limit3
            end
        else
            StopLimit3Valid = false
        end
    
    
    elseif OffsetType == "NoOffset" then
        
        if Amount1 > 0 then
            StopLevel1 = Stop1
            LimitLevel1 = Limit1
        else
            StopLimit1Valid = false
        end
        
        if Amount2 > 0 then
            StopLevel2 = Stop2
            LimitLevel2 = Limit2
        else
            StopLimit2Valid = false
        end
        
        if Amount3 > 0 then
            StopLevel3 = Stop3
            LimitLevel3 = Limit3
        else
            StopLimit3Valid = false
        end
        
    end
    StopLimitLevelsSet = true;    
end


function checkValidity()
    if Direction == "B" then
        if StopLimit1Valid then
            assert(StopLevel1 <= instance.bid[NOW], "The current bid price is " .. tostring(instance.bid[NOW]) .. ". This is already beyond the price set for Stop1 (" .. tostring(StopLevel1) .. "). Please set Stop1 below the current bid price of " .. tostring(instance.bid[NOW]) .. " and restart the strategy.");
            assert(LimitLevel1 >= instance.bid[NOW], "The current bid price is " .. tostring(instance.bid[NOW]) .. ". This is already beyond the price set for Limit1 (" .. tostring(LimitLevel1) .. "). Please set Limit2 above the current bid price of " .. tostring(instance.bid[NOW]) .. " and restart the strategy.");
        end
        
        if StopLimit2Valid then
            assert(StopLevel2 <= instance.bid[NOW], "The current bid price is " .. tostring(instance.bid[NOW]) .. ". This is already beyond the price set for Stop2 (" .. tostring(StopLevel2) .. "). Please set Stop2 below the current bid price of " .. tostring(instance.bid[NOW]) .. " and restart the strategy.");
            assert(LimitLevel2 >= instance.bid[NOW], "The current bid price is " .. tostring(instance.bid[NOW]) .. ". This is already beyond the price set for Limit2 (" .. tostring(LimitLevel2) .. "). Please set Limit2 above the current bid price of " .. tostring(instance.bid[NOW]) .. " and restart the strategy.");
        end
        
        if StopLimit3Valid then
            assert(StopLevel3 <= instance.bid[NOW], "The current bid price is " .. tostring(instance.bid[NOW]) .. ". This is already beyond the price set for Stop3 (" .. tostring(StopLevel3) .. "). Please set Stop3 below the current bid price of " .. tostring(instance.bid[NOW]) .. " and restart the strategy.");
            assert(LimitLevel3 >= instance.bid[NOW], "The current bid price is " .. tostring(instance.bid[NOW]) .. ". This is already beyond the price set for Limit3 (" .. tostring(LimitLevel3) .. "). Please set Limit2 above the current bid price of " .. tostring(instance.bid[NOW]) .. " and restart the strategy.");
        end
    else
        if StopLimit1Valid then
            assert(StopLevel1 >= instance.ask[NOW], "The current ask price is " .. tostring(instance.ask[NOW]) .. ". This is already beyond the price set for Stop1 (" .. tostring(StopLevel1) .. "). Please set Stop1 above the current ask price of " .. tostring(instance.ask[NOW]) .. " and restart the strategy.");
            assert(LimitLevel1 <= instance.ask[NOW], "The current ask price is " .. tostring(instance.ask[NOW]) .. ". This is already beyond the price set for Limit1 (" .. tostring(LimitLevel1) .. "). Please set Limit1 above the current ask price of " .. tostring(instance.ask[NOW]) .. " and restart the strategy.");
        end
        
        if StopLimit2Valid then
            assert(StopLevel2 >= instance.ask[NOW], "The current ask price is " .. tostring(instance.ask[NOW]) .. ". This is already beyond the price set for Stop2 (" .. tostring(StopLevel2) .. "). Please set Stop2 above the current ask price of " .. tostring(instance.ask[NOW]) .. " and restart the strategy.");
            assert(LimitLevel2 <= instance.ask[NOW], "The current ask price is " .. tostring(instance.ask[NOW]) .. ". This is already beyond the price set for Limit2 (" .. tostring(LimitLevel2) .. "). Please set Limit2 above the current ask price of " .. tostring(instance.ask[NOW]) .. " and restart the strategy.");
        end
        
        if StopLimit3Valid then
            assert(StopLevel3 >= instance.ask[NOW], "The current ask price is " .. tostring(instance.ask[NOW]) .. ". This is already beyond the price set for Stop3 (" .. tostring(StopLevel3) .. "). Please set Stop3 above the current ask price of " .. tostring(instance.ask[NOW]) .. " and restart the strategy.");
            assert(LimitLevel3 <= instance.ask[NOW], "The current ask price is " .. tostring(instance.ask[NOW]) .. ". This is already beyond the price set for Limit3 (" .. tostring(LimitLevel3) .. "). Please set Limit3 above the current ask price of " .. tostring(instance.ask[NOW]) .. " and restart the strategy.");
        end
    
    
    
    end


end



-- -----------------------------------------------------------------------
-- Return count of opened trades for spicific direction
-- (both directions if BuySell parameters is 'nil')
-- -----------------------------------------------------------------------
function checkSizeAndDirection()
    local enum, row;
    local lot = 0;
    enum = core.host:findTable("trades"):enumerator();
    row = enum:next();
    while row ~= nil do
        if row.AccountID == Account and
            row.OfferID == Offer and
            row.BS == Direction then
            lot = lot + row.AmountK;
        end
        row = enum:next();
    end

    return lot;
end


-- Close out portion of position
function exit(Amount)

    local AvailableLots = 0
    
    -- update TicketID and see if there is enough lots
    if CanClose then
        -- update TicketID if UK account
        checkTradeID();
        -- assigns correct number of lots if FX or CFD
        if Type == 1 then
            AvailableLots = Trade.AmountK
        else
            AvailableLots = Trade.AmountK * 1000
        end
    else
        -- make sure position size and direction is still accurate for US accounts
        AvailableLots = checkSizeAndDirection();
    end
    
    if AvailableLots >= Amount then

        local valuemap = core.valuemap();

        valuemap.Command = "CreateOrder";

        if not CanClose then
            -- FIFO Accounts
            valuemap.OrderType = "OM";
        else 
            -- Non-FIFO Accounts
            valuemap.OrderType = "CM";
            valuemap.TradeID = Trade.TradeID;
        end
        
        valuemap.OfferID = Offer;

        --core.host:trace("Closing Portion of Trade ID:" .. TradeID);

        if Direction == "B" then
            valuemap.BuySell = "S";
        else
            valuemap.BuySell = "B";
        end

        valuemap.Quantity = Amount * BaseSize;
        valuemap.AcctID = Account;
        valuemap.CustomID = "PSS_MTS";

        local success, msg = terminal:execute(100, valuemap);
        
        if not (success) then
           terminal:alertMessage(instance.bid:instrument(), instance.bid[NOW], "Create closing order failed:" .. msg, instance.bid:date(NOW));
        end
    else
        assert(false, "Attempting to close " .. tostring(Amount) .. " lot(s) when only " .. tostring(AvailableLots) .. " lot(s) are open. Stopping Strategy.");
    
    end

end


function ExtAsyncOperationFinished(id, success, message)
    --Get the orderID when order is successfully sent
   if id == 100 then
       if not success then
            logger:error("Closing portion of trade failed." .. message);
            core.host:trace("Closing portion of trade failed." .. message);
       else
            logger:info("Portion closed successfully: " .. message);
            core.host:trace("Portion closed successfully: " .. message);
       end
       Executing = false;
   end
end


-- Logging API
local function getLogPath()local path, fileName, fileHandle, cmd;path=os.getenv("USERPROFILE");if path~=nil then if string.find(path, "Users")~=
nil then path = path .. "\\Documents\\Marketscope";else path = path .. "\\Marketscope";end fileName=string.format("%s\\install.txt", path);
fileHandle=io.open(fileName, "w");if fileHandle==nil then cmd=string.format("MKDIR %s", path);os.execute(cmd);fileHandle = io.open(fileName, "w");
end if fileHandle~=nil then fileHandle:close();end end return path;end Logger={["OFF"]=0,["ERROR"]=1,["INFO"]=2,["DEBUG"]=3};Logger.__index
=Logger;function Logger:create(fileName,logLevel)assert(fileName~=nil,"Invalid log file name");assert(logLevel~=nil,"Invalid log level");assert(type(logLevel)
=="number","Invalid log level");assert(logLevel==Logger.OFF or logLevel==Logger.INFO or logLevel==Logger.DEBUG or logLevel==Logger.ERROR,
"Invalid log level");local logger={_logPath=getLogPath().."\\"..fileName..".txt",_logLevel=logLevel};setmetatable(logger,self);return logger;end function 
Logger:getLogPath()return self._logPath;end function Logger:writeLog(level,msg)local file=io.open(self._logPath,"a+");if file~=nil then file:write(os.date()
.."|"..level.."|"..msg.."\n");file:close();end end function Logger:info(msg)if self._logLevel>=Logger.INFO then self:writeLog("INFO",msg);end end 
function Logger:error(msg)if self._logLevel>=Logger.ERROR then self:writeLog("ERROR", msg);end end function Logger:debug(msg)if self._logLevel
>=Logger.DEBUG then self:writeLog("DEBUG", msg);end end

dofile(core.app_path() .. "\\strategies\\standard\\include\\helper.lua");


