<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE xsl:stylesheet [
  <!ENTITY nl "&#xa;">
]>
<xsl:stylesheet
    version="2.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:t="urn:templates"
    xmlns:str="http://exslt.org/strings"
    xmlns:ms="urn:schemas-microsoft-com:xslt"
    xmlns:gs="java://com.gehtsoft.js2ast.StringHelper">
  <xsl:output method="text" encoding="UTF-8" indent="no"/>

  <xsl:template name="string-replace-all">
    <xsl:param name="text" />
    <xsl:param name="replace" />
    <xsl:param name="by" />
    <xsl:choose>
        <xsl:when test="$text = '' or $replace = ''or not($replace)" >
            <xsl:value-of select="$text" />
        </xsl:when>
        <xsl:when test="contains($text, $replace)">
            <xsl:value-of select="substring-before($text,$replace)" />
            <xsl:value-of select="$by" />
            <xsl:call-template name="string-replace-all">
                <xsl:with-param name="text" select="substring-after($text,$replace)" />
                <xsl:with-param name="replace" select="$replace" />
                <xsl:with-param name="by" select="$by" />
            </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
            <xsl:value-of select="$text" />
        </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- xsl:template name="NewStringSpaces">
    <xsl:param name="prevPos"/>
    <xsl:param name="pos"/>

    <xsl:if test="$prevPos &lt; $pos">
      <xsl:text> </xsl:text>
      <xsl:call-template name="NewStringSpaces">
        <xsl:with-param name="prevPos" select="$prevPos + 1"/>
        <xsl:with-param name="pos" select="$pos"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template -->

  <xsl:template name="NewString">
    <xsl:param name="newString"/>
    <xsl:param name="pos"/>
    <xsl:variable name="lastString" select="gs:getLastString()"/>
    <xsl:if test="$lastString &lt; $newString">
      <xsl:variable name="tmp" select="gs:setLastString($lastString + 1)"/><xsl:text>
</xsl:text>
      <!-- xsl:call-template name="NewStringSpaces">
        <xsl:with-param name="pos" select="$pos"/>
        <xsl:with-param name="prevPos" select="0"/>
      </xsl:call-template -->
      <xsl:call-template name="NewString">
        <xsl:with-param name="prevString" select="$tmp"/>
        <xsl:with-param name="newString" select="$newString"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:variable name="templates" select="document('')//t:templates" />

  <t:templates>
    <t:CONSTANT/>
    <t:BINARYOPERATION/>
    <t:UNARYOPERATION/>
    <t:PARENTHESIS/>
    <t:POSTFIXUNARYOPERATION/>
    <t:TERNARYOPERATION/>
    <t:BLOCK/>
    <t:FUNCTIONDECLARATION/>
    <t:PARAMETERLIST/>
    <t:ID/>
    <t:BODY/>
    <t:EXPRESSIONLIST/>
    <t:CALLEXPRESSION/>
    <t:MEMBEREXPRESSION/>
    <t:VAR/>
    <t:VARLIST/>
    <t:IF/>
    <t:THEN/>
    <t:ELSE/>
    <t:RETURN/>
    <t:ARRAYEXPRESSION/>
    <t:THROW/>
    <t:STATEMENT/>
    <t:FOR_LOOP/>
    <t:FOR_INITIALIZER/>
    <t:FOR_CONDITION/>
    <t:FOR_AFTERTHOUGHT/>
    <t:NEWOBJECT/>
    <t:PROPERTY/>
    <t:TRY/>
    <t:CATCH/>
    <t:FINALLY/>
    <t:FOR_IN_LOOP/>
    <t:ITERATOR/>
    <t:ITERABLE_OBJECT/>
    <t:SWITCH/>
    <t:CONTINUE/>
    <t:BREAK/>
    <t:NEWARRAY/>
    <t:WHILE_LOOP/>
    <t:CONDITION/>
    <t:FUNCTIONEXPRESSION/>
    <t:DO_LOOP/>
    <t:EMPTYSTATEMENT/>
    <t:NEW/>
    <t:THIS/>
  </t:templates>

  <xsl:template name="CONSTANT" match="t:CONSTANT" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:param name="framing_number"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!CONSTANT</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:variable name="value">
      <xsl:choose>
        <xsl:when test="$node/@type = 'Number' and $framing_number='true'">[<xsl:value-of select="$node/@value"/>]</xsl:when>
        <xsl:when test="$node/@type = 'Null' or $node/@type = 'Undefined'">nil</xsl:when>
        <xsl:when test="$node/@type = 'RegularExpression'">GS_CREATE_REGULAR_EXPRESSION("<xsl:value-of select="$node/@value"/>", "<xsl:value-of select="$node/@flags"/>")</xsl:when>
        <xsl:otherwise><xsl:value-of select="$node/@value"/></xsl:otherwise>
      </xsl:choose>
    </xsl:variable><xsl:value-of disable-output-escaping="yes" select="$value"/><xsl:message>end!CONSTANT</xsl:message></xsl:template>

  <xsl:template name="BINARYOPERATION" match="t:BINARYOPERATION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!BINARYOPERATION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:variable name="oper1" select="$node/*[1]"/>
    <xsl:variable name="oper2" select="$node/*[2]"/>
    <xsl:variable name="operName1" select="name($oper1)"/>
    <xsl:variable name="operName2" select="name($oper2)"/>
    <xsl:choose>
      <xsl:when test="$node/@operation = 'Addition'">BinaryOperationAddition(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'Subtraction'">BinaryOperationSubtraction(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'Multiplication'">BinaryOperationMultiplication(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'Division'">BinaryOperationDivision(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'Modulus'">BinaryOperationModulus(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'Equal'">BinaryOperationEqual(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'EqualValueType'">BinaryOperationEqualValueType(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'NotEqual'">BinaryOperationNotEqual(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'NotEqualValueType'">BinaryOperationNotEqualValueType(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'LessThan'">BinaryOperationLessThan(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'LessEqualThan'">BinaryOperationLessEqualThan(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'GreaterThan'">BinaryOperationGreaterThan(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'GreaterEqualThan'">BinaryOperationGreaterEqualThan(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'And'"><xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates><xsl:text> and </xsl:text><xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates></xsl:when>
<xsl:when test="$node/@operation = 'Or'"><xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates><xsl:text> or </xsl:text><xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates></xsl:when>
<xsl:when test="$node/@operation = 'Assignment'"><xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>= <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates></xsl:when>
<xsl:when test="$node/@operation = 'AssignmentMultiplication'"><xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates> = BinaryOperationMultiplication(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'AssignmentDivision'"><xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates> = BinaryOperationDivision(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'AssignmentModulus'"><xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>= BinaryOperationModulus(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'AssignmentSubtraction'"><xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>= BinaryOperationSubtraction(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when>
<xsl:when test="$node/@operation = 'AssignmentAddition'"><xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>= BinaryOperationAddition(<xsl:apply-templates select="$templates/t:*[local-name() = $operName1]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper1"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>, <xsl:apply-templates select="$templates/t:*[local-name() = $operName2]"
                         mode="call-template">
            <xsl:with-param name="node" select="$oper2"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)</xsl:when></xsl:choose><xsl:message>end!BINARYOPERATION</xsl:message></xsl:template>

  <xsl:template name="UNARYOPERATION" match="t:UNARYOPERATION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!UNARYOPERATION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:choose><xsl:when test="$node/@operation = 'PlusOne'">GS_CALL_HELPER(function()
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates> = <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates> + 1;
    return <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>;
    end)</xsl:when>
<xsl:when test="$node/@operation = 'MinusOne'">GS_CALL_HELPER(function()
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates> = <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates> - 1;
    return <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>;
    end)</xsl:when>
<xsl:when test="$node/@operation = 'Plus'">UnaryOperationPlus(<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> )</xsl:when>
<xsl:when test="$node/@operation = 'Minus'">UnaryOperationMinus(<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> )</xsl:when>
<xsl:when test="$node/@operation = 'LogicalNot'">not (<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> )</xsl:when>
<xsl:when test="$node/@operation = 'BinaryNot'">bit.bnot(<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> )</xsl:when>
<xsl:when test="$node/@operation = 'Delete'"><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> = nil</xsl:when>
<xsl:when test="$node/@operation = 'Void'">UnaryOperationVoid(<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> )</xsl:when>
<xsl:when test="$node/@operation = 'TypeOf'">type(<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> )</xsl:when></xsl:choose><xsl:message>end!UNARYOPERATION</xsl:message></xsl:template>

  <xsl:template name="PARENTHESIS" match="t:PARENTHESIS" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:param name="delimeter"/><xsl:message>start!<xsl:value-of select="$node/@line"/>!PARENTHESIS</xsl:message>(<xsl:for-each select="$node/*">
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
        <xsl:variable name="currentNode" select="name(.)"/>
        <xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="delimeter" select="$delimeter"/>
        </xsl:apply-templates>
    </xsl:for-each>)<xsl:message>end!PARENTHESIS</xsl:message></xsl:template>

  <xsl:template name="POSTFIXUNARYOPERATION" match="t:POSTFIXUNARYOPERATION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!POSTFIXUNARYOPERATION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:choose><xsl:when test="$node/@operation = 'PlusOne'">
GS_CALL_HELPER(function()
    local GS_TMP_RESULT = <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates><xsl:text>
    </xsl:text><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> = <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> + 1
    return GS_TMP_RESULT;
    end)</xsl:when>
<xsl:when test="$node/@operation = 'MinusOne'">
GS_CALL_HELPER(function()
    local GS_TMP_RESULT = <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates><xsl:text>
    </xsl:text><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> = <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
        </xsl:apply-templates> - 1
    return GS_TMP_RESULT;
end)</xsl:when>
</xsl:choose>
<xsl:message>end!POSTFIXUNARYOPERATION</xsl:message></xsl:template>

  <xsl:template name="TERNARYOPERATION" match="t:TERNARYOPERATION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!TERNARYOPERATION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    GS_CALL_HELPER(function()
    if <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                             mode="call-template">
                <xsl:with-param name="node" select="$node/*[1]"/>
                <xsl:with-param name="prevString" select="$node/@line"/>
                <xsl:with-param name="isExpression" select="'in'"/>
            </xsl:apply-templates> then
        do return <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                             mode="call-template">
                <xsl:with-param name="node" select="$node/*[2]"/>
                <xsl:with-param name="prevString" select="$node/@line"/>
            </xsl:apply-templates> end
    else
        do return <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[3])]"
                             mode="call-template">
                <xsl:with-param name="node" select="$node/*[3]"/>
                <xsl:with-param name="prevString" select="$node/@line"/>
            </xsl:apply-templates> end
    end
end)<xsl:message>end!TERNARYOPERATION</xsl:message></xsl:template>

  <xsl:template name="BLOCK" match="t:BLOCK" mode="call-template">
    <xsl:param name="node"/><xsl:param name="prevString"/><xsl:message>start!<xsl:value-of select="$node/@line"/>!BLOCK</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:text>
</xsl:text>do<xsl:text>
</xsl:text><xsl:for-each select="$node/*">
        <xsl:variable name="currentNode" select="name(.)"/>
        <xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>
    </xsl:for-each><xsl:text>
</xsl:text>end
<xsl:message>end!BLOCK</xsl:message></xsl:template>

  <xsl:template name="FUNCTIONDECLARATION" match="t:FUNCTIONDECLARATION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!FUNCTIONDECLARATION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    function <xsl:value-of select="$node/@name"/>(<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                     mode="call-template">
        <xsl:with-param name="node" select="$node/*[1]"/>
        <xsl:with-param name="prevString" select="$node/@line"/>
      </xsl:apply-templates>)<xsl:text>
    </xsl:text><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                     mode="call-template">
        <xsl:with-param name="node" select="$node/*[2]"/>
        <xsl:with-param name="prevString" select="$node/@line"/>
      </xsl:apply-templates><xsl:text>
    </xsl:text>end
  <xsl:message>end!FUNCTIONDECLARATION</xsl:message></xsl:template>

  <xsl:template name="PARAMETERLIST" match="t:PARAMETERLIST" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!PARAMETERLIST</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:variable name="childCount" select="count($node/*)"/>
    <xsl:choose>
      <xsl:when test="$childCount=0"></xsl:when>
      <xsl:when test="$childCount=1"><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
          </xsl:apply-templates></xsl:when>
      <xsl:otherwise><xsl:for-each select="$node/*"><xsl:variable name="currentNode" select="name(.)"/><xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates><xsl:if test="position() != last()" >, </xsl:if></xsl:for-each></xsl:otherwise>
    </xsl:choose><xsl:message>end!PARAMETERLIST</xsl:message></xsl:template>

  <xsl:template name="ID" match="t:ID" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!ID</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:value-of select="$node/@name"/><xsl:message>end!ID</xsl:message></xsl:template>
  
  <xsl:template name="BODY" match="t:BODY" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!BODY</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:for-each select="$node/*"><xsl:variable name="currentNode" select="name(.)"/><xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="delimeter">; </xsl:with-param>
        </xsl:apply-templates>
    </xsl:for-each><xsl:message>end!BODY</xsl:message></xsl:template>

  <xsl:template name="EXPRESSIONLIST" match="t:EXPRESSIONLIST" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:param name="isExpression"/>
    <xsl:param name="delimeter"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!EXPRESSIONLIST</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:choose>
      <xsl:when test="$isExpression = 'before'"><xsl:for-each select="$node/*"><xsl:if test="position() != last()" ><xsl:variable name="currentNode" select="name(.)"/><xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                           mode="call-template">
              <xsl:with-param name="node" select="."/>
              <xsl:with-param name="prevString" select="$node/@line"/>
          </xsl:apply-templates><xsl:if test="position() != last() - 1" ><xsl:value-of select="$delimeter"/></xsl:if></xsl:if></xsl:for-each>
      </xsl:when>
      <xsl:when test="$isExpression = 'in'"><xsl:variable name="currentNode" select="name($node/*[position()=last()])"/><xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                           mode="call-template">
              <xsl:with-param name="node" select="$node/*[position()=last()]"/>
              <xsl:with-param name="prevString" select="$node/@line"/>
          </xsl:apply-templates>
      </xsl:when>
      <xsl:otherwise>
      <xsl:variable name="childCount" select="count($node/*)"/>
      <xsl:choose>
        <xsl:when test="$childCount=0"></xsl:when>
        <xsl:when test="$childCount=1"><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                           mode="call-template">
              <xsl:with-param name="node" select="$node/*[1]"/>
              <xsl:with-param name="prevString" select="$node/@line"/>
            </xsl:apply-templates></xsl:when>
        <xsl:otherwise><xsl:for-each select="$node/*"><xsl:variable name="currentNode" select="name(.)"/><xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                           mode="call-template">
              <xsl:with-param name="node" select="."/>
              <xsl:with-param name="prevString" select="$node/@line"/>
          </xsl:apply-templates><xsl:if test="position() != last()" ><xsl:value-of select="$delimeter"/></xsl:if></xsl:for-each></xsl:otherwise>
      </xsl:choose></xsl:otherwise></xsl:choose><xsl:message>end!EXPRESSIONLIST</xsl:message></xsl:template>

  <xsl:template name="CALLEXPRESSION" match="t:CALLEXPRESSION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!CALLEXPRESSION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[1]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
      <xsl:with-param name="isCallExpression" select="'true'"/>
      <xsl:with-param name="delimeter">:</xsl:with-param>
    </xsl:apply-templates>(<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[2]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
      <xsl:with-param name="delimeter">, </xsl:with-param>
    </xsl:apply-templates>)<xsl:message>end!CALLEXPRESSION</xsl:message></xsl:template>

  <xsl:template name="MEMBEREXPRESSION" match="t:MEMBEREXPRESSION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:param name="isCallExpression"/>
    <xsl:param name="delimeter"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!MEMBEREXPRESSION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[1]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
    </xsl:apply-templates><xsl:choose>
      <xsl:when test="$isCallExpression != ''"><xsl:value-of select="$delimeter"/></xsl:when>
      <xsl:otherwise>.</xsl:otherwise>
      </xsl:choose><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[2]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
    </xsl:apply-templates><xsl:message>end!MEMBEREXPRESSION</xsl:message></xsl:template>

  <xsl:template name="VAR" match="t:VAR" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!VAR</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>local <xsl:value-of select ="$node/@name"/><xsl:if test="count($node/*) != 0"> = <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[1]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
    </xsl:apply-templates></xsl:if><xsl:message>end!VAR</xsl:message></xsl:template>

  <xsl:template name="VARLIST" match="t:VARLIST" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!VARLIST</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:for-each select="$node/*">
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
        <xsl:variable name="currentNode" select="name(.)"/>
        <xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates><xsl:if test="position() != last()" >;</xsl:if>
    </xsl:for-each><xsl:message>end!VARLIST</xsl:message></xsl:template>

  <xsl:template name="IF" match="t:IF" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!IF</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="isExpression" select="'before'"/>
            <xsl:with-param name="delimeter">; </xsl:with-param>
        </xsl:apply-templates>
    if <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="isExpression" select="'in'"/>
        </xsl:apply-templates><xsl:text> </xsl:text>then<xsl:text>
</xsl:text><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[2]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates><xsl:text>
</xsl:text>else<xsl:text>
</xsl:text><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[3])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[3]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates><xsl:text>
</xsl:text>end
  <xsl:message>end!IF</xsl:message></xsl:template>

  <xsl:template name="THEN" match="t:THEN" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!THEN</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[1]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
      <xsl:with-param name="isCallExpression" select="'true'"/>
      <xsl:with-param name="delimeter">:</xsl:with-param>
    </xsl:apply-templates><xsl:message>end!THEN</xsl:message></xsl:template>

  <xsl:template name="ELSE" match="t:ELSE" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!ELSE</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[1]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
      <xsl:with-param name="isCallExpression" select="'true'"/>
      <xsl:with-param name="delimeter">:</xsl:with-param>
    </xsl:apply-templates><xsl:message>end!ELSE</xsl:message></xsl:template>

  <xsl:template name="RETURN" match="t:RETURN" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!RETURN</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    do return <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates> end
  <xsl:message>end!RETURN</xsl:message></xsl:template>

  <xsl:template name="ARRAYEXPRESSION" match="t:ARRAYEXPRESSION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!ARRAYEXPRESSION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>[<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[2]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>]<xsl:message>end!ARRAYEXPRESSION</xsl:message></xsl:template>

  <xsl:template name="THROW" match="t:THROW" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/><xsl:message>start!<xsl:value-of select="$node/@line"/>!THROW</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>error(<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)
  <xsl:message>end!THROW</xsl:message></xsl:template>

  <xsl:template name="STATEMENT" match="t:STATEMENT" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/><xsl:message>start!<xsl:value-of select="$node/@line"/>!STATEMENT</xsl:message><xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:for-each select="$node/*">
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
        <xsl:variable name="currentNode" select="name(.)"/>
        <xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>;
    </xsl:for-each><xsl:message>end!STATEMENT</xsl:message></xsl:template>

  <xsl:template name="FOR_LOOP" match="t:FOR_LOOP" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!FOR_LOOP</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates><xsl:text>
    </xsl:text><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[2]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="isExpression" select="'before'"/>
            <xsl:with-param name="delimeter">; </xsl:with-param>
        </xsl:apply-templates>
    function ForLoop_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>()
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[4])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[4]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>
        return GS_LOOP_TABLE_CONTINUE
    end

    while <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[2]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="isExpression" select="'in'"/>
        </xsl:apply-templates> do
        GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> = ForLoop_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>()

        if  GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> == GS_LOOP_TABLE_BREAK then
            break;
        else
            if GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> ~= GS_LOOP_TABLE_CONTINUE then
                do return GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> end
            end
        end

        <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[3])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[3]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>    
    
        <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[2]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="isExpression" select="'before'"/>
            <xsl:with-param name="delimeter">; </xsl:with-param>
        </xsl:apply-templates>
    end
  <xsl:message>end!FOR_LOOP</xsl:message></xsl:template>

  <xsl:template name="FOR_INITIALIZER" match="t:FOR_INITIALIZER" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!FOR_INITIALIZER</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:for-each select="$node/*">
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
        <xsl:variable name="currentNode" select="name(.)"/>
        <xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>
    </xsl:for-each><xsl:message>end!FOR_INITIALIZER</xsl:message></xsl:template>

  <xsl:template name="FOR_CONDITION" match="t:FOR_CONDITION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:param name="isExpression"/><xsl:message>start!<xsl:value-of select="$node/@line"/>!FOR_CONDITION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:for-each select="$node/*">
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
        <xsl:variable name="currentNode" select="name(.)"/>
        <xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="isExpression" select="$isExpression"/>
        </xsl:apply-templates>
    </xsl:for-each><xsl:message>end!FOR_CONDITION</xsl:message></xsl:template>

  <xsl:template name="FOR_AFTERTHOUGHT" match="t:FOR_AFTERTHOUGHT" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!FOR_AFTERTHOUGHT</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:for-each select="$node/*">
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
        <xsl:variable name="currentNode" select="name(.)"/>
        <xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>
    </xsl:for-each><xsl:message>end!FOR_AFTERTHOUGHT</xsl:message></xsl:template>

  <xsl:template name="NEWOBJECT" match="t:NEWOBJECT" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!NEWOBJECT</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>{
    <xsl:for-each select="$node/PROPERTY"><xsl:variable name="currentNode" select="name(.)"/><xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates><xsl:if test="position() != last()">, </xsl:if></xsl:for-each>
    }<xsl:message>end!NEWOBJECT</xsl:message></xsl:template>

  <xsl:template name="PROPERTY" match="t:PROPERTY" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!PROPERTY</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="framing_number" select="'true'"/>
    </xsl:apply-templates><xsl:if test="$node/*[2]"> = <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[2]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
    </xsl:apply-templates></xsl:if><xsl:message>end!PROPERTY</xsl:message></xsl:template>

  <xsl:template name="TRY" match="t:TRY" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!TRY</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    do
    local GS_RESULT_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> = pcall(function()

    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>

    end)
    if not GS_RESULT_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> then

    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[2]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>

    end
    end
  <xsl:message>end!TRY</xsl:message></xsl:template>

  <xsl:template name="CATCH" match="t:CATCH" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!CATCH</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[1]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
      <xsl:with-param name="isCallExpression" select="'true'"/>
      <xsl:with-param name="delimeter">:</xsl:with-param>
    </xsl:apply-templates><xsl:message>end!CATCH</xsl:message></xsl:template>

  <xsl:template name="FINALLY" match="t:FINALLY" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!FINALLY</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[1]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
      <xsl:with-param name="isCallExpression" select="'true'"/>
      <xsl:with-param name="delimeter">:</xsl:with-param>
    </xsl:apply-templates><xsl:message>end!FINALLY</xsl:message></xsl:template>

  <xsl:template name="FOR_IN_LOOP" match="t:FOR_IN_LOOP" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!FOR_IN_LOOP</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>

function ForInLoop_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>(
<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>  
)
    
<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[3])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[3]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>  

    return GS_LOOP_TABLE_CONTINUE
    end
for GS_HELPER_ITERATOR_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>1, GS_HELPER_ITERATOR_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>2 in pairs(<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[2]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>) do
    local GS_HELPER_CALL_<xsl:value-of select="$node/@line"/> = ForInLoop_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>(GS_HELPER_ITERATOR_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>1, GS_HELPER_ITERATOR_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>2)
   
    if GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> == GS_LOOP_TABLE_BREAK then
            break
        else
            if GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> ~= GS_LOOP_TABLE_CONTINUE then
                do return GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> end 
            end
        end
    end
  <xsl:message>end!FOR_IN_LOOP</xsl:message></xsl:template>

  <xsl:template name="ITERATOR" match="t:ITERATOR" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!ITERATOR</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[1]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
      <xsl:with-param name="isCallExpression" select="'true'"/>
      <xsl:with-param name="delimeter">:</xsl:with-param>
    </xsl:apply-templates>

  <xsl:message>end!ITERATOR</xsl:message></xsl:template>

  <xsl:template name="ITERABLE_OBJECT" match="t:ITERABLE_OBJECT" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!ITERABLE_OBJECT</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/*[1]"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
      <xsl:with-param name="isCallExpression" select="'true'"/>
      <xsl:with-param name="delimeter">:</xsl:with-param>
    </xsl:apply-templates><xsl:message>end!ITERABLE_OBJECT</xsl:message></xsl:template>

  <xsl:template name="SWITCH" match="t:SWITCH" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!SWITCH</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:text>
  </xsl:text>function switch_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>()<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="isExpression" select="'before'"/>
            <xsl:with-param name="delimeter">; </xsl:with-param>
        </xsl:apply-templates><xsl:text>
      </xsl:text>
        <xsl:variable name="condition_orig"><xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="isExpression" select="'in'"/>
        </xsl:apply-templates></xsl:variable>
local GS_CONDITION_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> = <xsl:value-of select="$condition_orig"/><xsl:text>
</xsl:text>local GS_CONDITION_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>_ORIG = GS_CONDITION_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/><xsl:text>
</xsl:text>
<xsl:for-each select="$node/CASE"><xsl:text>
</xsl:text><xsl:variable name="case" select="."/>function GS_CASE_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/><xsl:text>_</xsl:text><xsl:value-of select="position()"/>()<xsl:for-each select="./*"><xsl:if test="position() != 1"><xsl:variable name="currentNode" select="name(.)"/><xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                           mode="call-template">
              <xsl:with-param name="node" select="."/>
              <xsl:with-param name="prevString" select="$case/@line"/>
          </xsl:apply-templates><xsl:text>
        </xsl:text></xsl:if>
</xsl:for-each><xsl:text>
</xsl:text>return GS_LOOP_TABLE_CONTINUE
end</xsl:for-each>

<xsl:for-each select="$node/CASE"><xsl:variable name="case" select="."/><xsl:variable name="expr" select="./*[1]"/><xsl:text>
</xsl:text><xsl:apply-templates select="$templates/t:*[local-name() = name($expr)]"
                         mode="call-template">
            <xsl:with-param name="node" select="$expr"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="isExpression" select="'before'"/>
            <xsl:with-param name="delimeter">; </xsl:with-param>
        </xsl:apply-templates><xsl:text>
</xsl:text>
if BinaryOperationEqual(GS_CONDITION_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>, <xsl:apply-templates select="$templates/t:*[local-name() = name($expr)]"
                 mode="call-template">
    <xsl:with-param name="node" select="$expr"/>
    <xsl:with-param name="prevString" select="./@line"/>
    <xsl:with-param name="isExpression" select="'in'"/>
</xsl:apply-templates>) then
  local GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/><xsl:text>_</xsl:text><xsl:value-of select="position()"/> = GS_CASE_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/><xsl:text>_</xsl:text><xsl:value-of select="position()"/>()
  if GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/><xsl:text>_</xsl:text><xsl:value-of select="position()"/> == GS_LOOP_TABLE_CONTINUE then
<xsl:choose>
<xsl:when test="position() = last()">do end</xsl:when>
<xsl:otherwise>
<xsl:variable name="subl" select="$case/following-sibling::*[1]/*[1]"/>GS_CONDITION_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> = <xsl:apply-templates select="$templates/t:*[local-name() = name($subl)]"
                 mode="call-template">
    <xsl:with-param name="node" select="$subl"/>
    <xsl:with-param name="prevString" select="$expr/@line"/>
    <xsl:with-param name="isExpression" select="'in'"/>
</xsl:apply-templates>
</xsl:otherwise>
</xsl:choose>
  else
    do return GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/><xsl:text>_</xsl:text><xsl:value-of select="position()"/> end
  end
end
</xsl:for-each>
<xsl:text>
</xsl:text>
<xsl:for-each select="$node/DEFAULT/*"><xsl:variable name="currentNode" select="name(.)"/><xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>
    </xsl:for-each><xsl:text>
  </xsl:text>
      return GS_LOOP_TABLE_BREAK
    end

  local GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> = switch_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/>();
  if (GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> ~= GS_LOOP_TABLE_BREAK) then
    do return GS_HELPER_CALL_<xsl:value-of select="$node/@line"/><xsl:text>_</xsl:text><xsl:value-of select="$node/@position"/> end
  end
<xsl:message>end!SWITCH</xsl:message></xsl:template>

  <xsl:template name="BREAK" match="t:BREAK" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!BREAK</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    do return GS_LOOP_TABLE_BREAK end
  <xsl:message>end!BREAK</xsl:message></xsl:template>

  <xsl:template name="CONTINUE" match="t:CONTINUE" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!CONTINUE</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    do return GS_LOOP_TABLE_CONTINUE end
  <xsl:message>end!CONTINUE</xsl:message></xsl:template>

   <xsl:template name="NEWARRAY" match="t:NEWARRAY" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!NEWARRAY</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>{<xsl:for-each select="$node/*"><xsl:variable name="currentNode" select="name(.)"/><xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates><xsl:if test="position() != last()">, </xsl:if>
    </xsl:for-each>}
  <xsl:message>end!NEWARRAY</xsl:message></xsl:template>

  <xsl:template name="WHILE_LOOP" match="t:WHILE_LOOP" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!WHILE_LOOP</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
function ForLoop_<xsl:value-of  select="$node/@line"/>_<xsl:value-of select="$node/@position"/>()
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[2]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>
    return GS_LOOP_TABLE_CONTINUE
end
while <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates> do
    GS_HELPER_CALL_<xsl:value-of  select="$node/@line"/>_<xsl:value-of select="$node/@position"/> = ForLoop_<xsl:value-of  select="$node/@line"/>_<xsl:value-of select="$node/@position"/>() 
    if GS_HELPER_CALL_<xsl:value-of  select="$node/@line"/>_<xsl:value-of select="$node/@position"/> == GS_LOOP_TABLE_BREAK then 
        break;
    else
        if GS_HELPER_CALL_<xsl:value-of  select="$node/@line"/>_<xsl:value-of select="$node/@position"/> ~= GS_LOOP_TABLE_CONTINUE then 
            do return GS_HELPER_CALL_<xsl:value-of  select="$node/@line"/>_<xsl:value-of select="$node/@position"/> end 
        end
    end
end
  <xsl:message>end!WHILE_LOOP</xsl:message></xsl:template>

  <xsl:template name="CONDITION" match="t:CONDITION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:param name="isExpression"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!CONDITION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template><xsl:for-each select="$node/*">
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
        <xsl:variable name="currentNode" select="name(.)"/>
        <xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="$node/@line"/>
            <xsl:with-param name="isExpression" select="$isExpression"/>
        </xsl:apply-templates>
    </xsl:for-each><xsl:message>end!CONDITION</xsl:message></xsl:template>

  <xsl:template name="FUNCTIONEXPRESSION" match="t:FUNCTIONEXPRESSION" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!FUNCTIONEXPRESSION</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    function (<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                     mode="call-template">
        <xsl:with-param name="node" select="$node/*[1]"/>
        <xsl:with-param name="prevString" select="$node/@line"/>
      </xsl:apply-templates>)
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                     mode="call-template">
        <xsl:with-param name="node" select="$node/*[2]"/>
        <xsl:with-param name="prevString" select="$node/@line"/>
      </xsl:apply-templates>
    end
  <xsl:message>end!FUNCTIONEXPRESSION</xsl:message></xsl:template>

  <xsl:template name="DO_LOOP" match="t:DO_LOOP" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!DO_LOOP</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
function ForLoop_<xsl:value-of select="$node/@line"/>_<xsl:value-of select="$node/@position"/>()
    <xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[2])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[2]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>    
    return GS_LOOP_TABLE_CONTINUE
end
repeat
    GS_HELPER_CALL_<xsl:value-of select="$node/@line"/>_<xsl:value-of select="$node/@position"/> = ForLoop_<xsl:value-of select="$node/@line"/>_<xsl:value-of select="$node/@position"/>() 
    if GS_HELPER_CALL_<xsl:value-of select="$node/@line"/>_<xsl:value-of select="$node/@position"/> == GS_LOOP_TABLE_BREAK then 
        break;
    else
        if GS_HELPER_CALL_<xsl:value-of select="$node/@line"/>_<xsl:value-of select="$node/@position"/> ~= GS_LOOP_TABLE_CONTINUE then 
            do return GS_HELPER_CALL_<xsl:value-of select="$node/@line"/>_<xsl:value-of select="$node/@position"/> end 
        end
    end
until not(<xsl:apply-templates select="$templates/t:*[local-name() = name($node/*[1])]"
                         mode="call-template">
            <xsl:with-param name="node" select="$node/*[1]"/>
            <xsl:with-param name="prevString" select="$node/@line"/>
        </xsl:apply-templates>)
  <xsl:message>end!DO_LOOP</xsl:message></xsl:template>

  <xsl:template name="THIS" match="t:THIS" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!THIS</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    self
  <xsl:message>end!THIS</xsl:message></xsl:template>

  <xsl:template name="NEW" match="t:NEW" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!NEW</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template> {}
    <xsl:value-of select="$node/CALLEXPRESSION/ID/@name"/>(<xsl:value-of select="$node/../@name"/><xsl:if test="$node/CALLEXPRESSION/EXPRESSIONLIST">, <xsl:apply-templates select="$templates/t:EXPRESSIONLIST"
                   mode="call-template">
      <xsl:with-param name="node" select="$node/CALLEXPRESSION/EXPRESSIONLIST"/>
      <xsl:with-param name="prevString" select="$node/@line"/>
      <xsl:with-param name="delimeter">, </xsl:with-param>
    </xsl:apply-templates></xsl:if>)
  <xsl:message>end!NEW</xsl:message></xsl:template>

  <xsl:template name="EMPTYSTATEMENT" match="t:EMPTYSTATEMENT" mode="call-template">
    <xsl:param name="node"/>
    <xsl:param name="prevString"/>
    <xsl:message>start!<xsl:value-of select="$node/@line"/>!EMPTYSTATEMENT</xsl:message>
    <xsl:call-template name="NewString">
      <xsl:with-param name="pos" select="$node/@position"/>
      <xsl:with-param name="newString" select="$node/@line"/>
    </xsl:call-template>
    <xsl:for-each select="$node/*">
      <xsl:call-template name="NewString">
        <xsl:with-param name="pos" select="$node/@position"/>
        <xsl:with-param name="newString" select="$node/@line"/>
      </xsl:call-template>
          <xsl:variable name="currentNode" select="name(.)"/>
          <xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                           mode="call-template">
              <xsl:with-param name="node" select="."/>
              <xsl:with-param name="prevString" select="$node/@line"/>
          </xsl:apply-templates>
    </xsl:for-each>
  <xsl:message>end!EMPTYSTATEMENT</xsl:message></xsl:template>

  <xsl:template match="/">
    dofile(core.app_path() .. "/jsMigration.lua")
    InitMigration()
    <xsl:for-each select="/PROGRAM/*"><xsl:variable name="currentNode" select="name(.)"/><xsl:apply-templates select="$templates/t:*[local-name() = $currentNode]"
                         mode="call-template">
            <xsl:with-param name="node" select="."/>
            <xsl:with-param name="prevString" select="0"/>
        </xsl:apply-templates>
    </xsl:for-each>
  </xsl:template>
</xsl:stylesheet>
